/***************************************************************
  TORNADO-54x utilities (C54x DSP Environment).
  (C) MicroLAB Systems Ltd, 1997-2001

****************************************************************/

#ifndef __T54X_DEF__
#define __T54X_DEF__	1


//**** recommended SWWSR/BSCR/SWCR/CLKMD register settings *******

#define T542L_SWWSR_DATA	0x2000	// ext RAM - 0ws, IO - 2ws
#define T548_SWWSR_DATA		0x2209  // ext RAM - 1ws, IO - 2ws
#define T549_SWWSR_DATA		0x2209  // ext RAM - 1ws, IO - 2ws
#define T5402_SWWSR_DATA	0x2249  // ext RAM - 1ws, IO - 2ws
#define T5410_SWWSR_DATA	0x2209	// ext RAM - 1ws, IO - 2ws
#define T5416_SWWSR_DATA	0x4412	// ext RAM - 2ws, IO - 4ws
#define T548_CLKMD_DATA		0x07f7	// CLKMD reg (PLLx1)
#define T549_CLKMD_DATA		0x07f7	// CLKMD reg (PLLx1)
#define T5402_CLKMD_DATA	0xf7f7	// CLKMD reg (PLLx1)
#define T5410_CLKMD_DATA	0x07f7	// CLKMD reg (PLLx1)
#define T5416_CLKMD_DATA	0x17f7	// CLKMD reg (PLLx2)
#define T548_BSCR_DATA		0x8800	// BSCR reg
#define T549_BSCR_DATA		0x8800	// BSCR reg
#define T5402_BSCR_DATA		0xc800	// BSCR reg
#define T5410_BSCR_DATA		0x8000	// BSCR reg
#define T5416_BSCR_DATA		0x8000	// BSCR reg
#define T549_SWCR_DATA		0x0000	// SWCR reg
#define T5402_SWCR_DATA		0x0000	// SWCR reg
#define T5410_SWCR_DATA		0x0000	// SWCR reg
#define T5416_SWCR_DATA		0x0000	// SWCR reg

#define _T54X_SWWSR_DATA	0x4452	// universal setting: ext RAM - 2ws, C5402/<8000K: 1ws, IO - 4ws
#define _T54X_BSCR_DATA		0x8000	// universal BSCR setting
//****************************************************************


#define	ON						1
#define	OFF						0

#define	T54X_XIOF_OUTPUT		1
#define	T54X_XIOF_INPUT			0


/* XIO pins configurator: T548/549/5402/5410/T5416 rev.2a and later */

#define	T54X_XIO_IO_PIN			0
#define	T54X_XIO_TM_PIN			1


/* PIOX/SIOX Reset Control: T54x rev.2C and later */

#define	T54X_PX_RESET_BITMASK	1
#define	T54X_PX_RESET_ON		0
#define	T54X_PX_RESET_OFF		1
#define	T54X_SXA_RESET_BITMASK	2
#define	T54X_SXA_RESET_ON		0
#define	T54X_SXA_RESET_OFF		2
#define	T54X_SXB_RESET_BITMASK	4
#define	T54X_SXB_RESET_ON		0
#define	T54X_SXB_RESET_OFF		4


//-- extended API functions declarations

void	T54X_init_dsp();
void	T542L_init_dsp();
void	T548_init_dsp();
void	T549_init_dsp();
void	T5402_init_dsp();
void	T5410_init_dsp();
void	T5416_init_dsp();
void	T54X_set_mh_rq();
void 	T54X_set_mlock(int v);
int 	T54X_mlock();
void 	T54X_set_xreset(int v);
int 	T54X_xreset();
void 	T54X_set_px_run(int v);
int 	T54X_px_run();
void 	T54X_set_sxa_run(int v);
int 	T54X_sxa_run();
void 	T54X_set_sxb_run(int v);
int 	T54X_sxb_run();
int 	T54X_pxsx_run();
void 	T54X_set_pxsx_run(int v);
void 	T54X_set_xiof0_dir(int dir);
void 	T54X_set_xiof1_dir(int dir);
int 	T54X_xiof0_dir();
int		T54X_xiof1_dir();
void 	T54X_set_xiof0(int v);
void 	T54X_set_xiof1(int v);
int 	T54X_xiof0();
int 	T54X_xiof1();
void 	T54X_set_xio0_cnf(int cnf);
void 	T54X_set_xio1_cnf(int cnf);
int 	T54X_xio0_cnf();
int 	T54X_xio1cnf();
void 	T54X_set_dmem_page(int p);
int 	T54X_dmem_page();



/***************** Extended T54x DSP API functions ************************
 Extended T54x DSP API functions include functions for accessing DSP IOX area.
 However, in order to reduce the size of DSP code and in order to exclude mutiple 
 function body appearance, any of the Extended API functions can be included/excluded 
 during the compilation process. Inclusion/exclusion of particular Extended API 
 function is controled via global vars, which can be defined in user code to perform 
 a particular action:

	__T54X_EXCLUDE_INIT_DSP__ 
		- if defined in user code, then the DSP INIT functions are excluded
		  from the user source code. Default is INCLUDED.

	__T54X_EXCLUDE_IOX__
		- if defined in user code, then the DSP IOX functions are excluded
		  from the user source code. Default is INCLUDED.
*/


#ifndef __T54X_EXCLUDE_INIT_DSP__
//=============== DSP init functions ===============================
// This function performs initialization of T54x DSP configiration registers
// for different T54X boards.


#define	__C54X_SWWSR__	0x28
#define	__C54X_BSCR__	0x29
#define	__C54X_SWCR__	0x2B
#define	__C54X_CLKMD__	0x58
#define	__C54X_HPIC__	0x2C


void	T54X_init_dsp()
// This fucntion can be used for general initialization of any T54X in
// to gain compatibility for all T54x DSP platforms.
// Note !!!:
//	This function uses the worst possible configuration values
// 	in order to provide max WS for functionality of all DSP environment 
//	across all T54x boards (XRAM: 2ws, I/O:3ws)
{ 
  *(unsigned *)__C54X_HPIC__  = 0x2;	
  *(unsigned *)__C54X_SWWSR__ = _T54X_SWWSR_DATA;
  *(unsigned *)__C54X_BSCR__ = _T54X_BSCR_DATA;
}



void	T542L_init_dsp()
// This function can be used for initialization of T542L on-board DSP.
{ 
  *(unsigned *)__C54X_HPIC__  = 0x2;	// enable shared mode for HPI
  *(unsigned *)__C54X_SWWSR__ = T542L_SWWSR_DATA;
}

void	T548_init_dsp()
// This function can be used for initialization of T548 on-board DSP.
{ 
  *(unsigned *)__C54X_HPIC__  = 0x2;	// enable shared mode for HPI
  *(unsigned *)__C54X_SWWSR__ = T548_SWWSR_DATA;
  *(unsigned *)__C54X_BSCR__ = T548_BSCR_DATA;
  *(unsigned *)__C54X_CLKMD__ = T548_CLKMD_DATA;
}

void	T549_init_dsp()
// This function can be used for initialization of T549 on-board DSP.
{ 
  *(unsigned *)__C54X_HPIC__  = 0x2;	// enable shared mode for HPI
  *(unsigned *)__C54X_SWWSR__ = T549_SWWSR_DATA;
  *(unsigned *)__C54X_SWCR__ = T549_SWCR_DATA;
  *(unsigned *)__C54X_BSCR__ = T549_BSCR_DATA;
  *(unsigned *)__C54X_CLKMD__ = T549_CLKMD_DATA;
}


void	T5402_init_dsp()
// This function can be used for initialization of T5402 on-board DSP.
{ 
  *(unsigned *)__C54X_HPIC__  = 0x2;	// enable shared mode for HPI
  *(unsigned *)__C54X_SWWSR__ = T5402_SWWSR_DATA;
  *(unsigned *)__C54X_SWCR__ = T5402_SWCR_DATA;
  *(unsigned *)__C54X_BSCR__ = T5402_BSCR_DATA;
  *(unsigned *)__C54X_CLKMD__ = T5402_CLKMD_DATA;
}

void	T5410_init_dsp()
// This function can be used for initialization of T5410 on-board DSP.
{ 
  *(unsigned *)__C54X_HPIC__  = 0x2;	// enable shared mode for HPI
  *(unsigned *)__C54X_SWWSR__ = T5410_SWWSR_DATA;
  *(unsigned *)__C54X_SWCR__ = T5410_SWCR_DATA;
  *(unsigned *)__C54X_BSCR__ = T5410_BSCR_DATA;
  *(unsigned *)__C54X_CLKMD__ = T5410_CLKMD_DATA;
}

void	T5416_init_dsp()
// This function can be used for initialization of T5416 on-board DSP.
{ 
  *(unsigned *)__C54X_HPIC__  = 0x2;	// enable shared mode for HPI
  *(unsigned *)__C54X_SWWSR__ = T5416_SWWSR_DATA;
  *(unsigned *)__C54X_SWCR__ = T5416_SWCR_DATA;
  *(unsigned *)__C54X_BSCR__ = T5416_BSCR_DATA;
  *(unsigned *)__C54X_CLKMD__ = T5416_CLKMD_DATA;
}

//======================================================================
#endif


#ifndef __T54X_EXCLUDE_IOX__
//=============== DSP IOX access functions ===============================
// These functions perform access to DSP IOX area.

ioport unsigned port8000;
ioport unsigned port8001;
ioport unsigned port8002;
ioport unsigned port4002;
ioport unsigned port4001;
ioport unsigned port4000;
ioport unsigned port2000;

// define SET XF macros, which must be used for access to T54x DSP IOX area
#define __C54X_SET_XF__		asm(" ssbx xf")


//************************* MLOCK & MH_RQ Control **************

void T54X_set_mh_rq()
{ 
 __C54X_SET_XF__;
 port8000=0;
}

void T54X_set_mlock(int v)
{
 __C54X_SET_XF__;
 if(v==OFF) port8001=0;
   else	    port8001=1;
}

int T54X_mlock()
{
 __C54X_SET_XF__;
 return(port8001 & 1);
}


/* XRESET control: T548/549/5402/5410 rev.2A/2B only
   (common PIOX/SIOX RESET signal)
   note: XRESET is generated as OR between DSP_RESET signal
	 and Bit-0 of port #8002
*/

void T54X_set_xreset(int v)
{
 __C54X_SET_XF__;
 if(v==OFF) port8002=0;
   else	    port8002=1;
}

int T54X_xreset()
{
 __C54X_SET_XF__;
 return(port8002 & 1);
}


/* PX/SX-RESET control: T54x rev.2C and later
   (separate PIOX/SIOX RESET signal)

   notes:
     1. PIOX RESET is generated as bit-0 of port #8002
	(==0: PIOX RESET is active, ==1: PIOX RESET is OFF).
	PIOX_RESET is set active on DSP RESET.
     2. SIOX-A RESET is generated as bit-1 of port #8002
	(==0: SIOX-A RESET is active, ==1: SIOX-A RESET is OFF).
	SIOX-A RESET is set active on DSP RESET.
     3. SIOX-B RESET is generated as bit-2 of port #8002
	(==0: SIOX-B RESET is active, ==1: SIOX-B RESET is OFF).
	SIOX-B RESET is set active on DSP RESET.
*/

void T54X_set_px_run(int v)
{
 __C54X_SET_XF__;
 if(v==ON) port8002 |=  T54X_PX_RESET_OFF; 
   else	   port8002 &= ~T54X_PX_RESET_BITMASK;
}

int T54X_px_run()
{
 __C54X_SET_XF__;
 return(port8002 & 1);
}


void T54X_set_sxa_run(int v)
{
 __C54X_SET_XF__;
 if(v==ON) port8002 |=  T54X_SXA_RESET_OFF; 
   else	   port8002 &= ~T54X_SXA_RESET_BITMASK;
}

int T54X_sxa_run()
{
 __C54X_SET_XF__;
 return(((port8002) >>1) &1);
}


void T54X_set_sxb_run(int v)
{
 __C54X_SET_XF__;
 if(v==ON) port8002 |=  T54X_SXB_RESET_OFF; 
   else	   port8002 &= ~T54X_SXB_RESET_BITMASK;
}

int T54X_sxb_run()
{
 __C54X_SET_XF__;
 return(((port8002) >>2) &1);
}


void T54X_set_pxsx_run(int v)
{
 __C54X_SET_XF__;
 port8002= (v & 7);
}

int T54X_pxsx_run()
{
 __C54X_SET_XF__;
 return(port8002 & 7);
}



/************************ XIOF Control ***************************/

void T54X_set_xiof0_dir(int dir)
{
 __C54X_SET_XF__;
 if(dir==T54X_XIOF_INPUT)
     port4001=port4001 & ~1;
   else
     port4001=port4001 | 1;
}

void T54X_set_xiof1_dir(int dir)
{
 __C54X_SET_XF__;
 if(dir==T54X_XIOF_INPUT)
     port4001=port4001 & ~2;
   else
     port4001=port4001 | 2;
}

int T54X_xiof0_dir()
{
 __C54X_SET_XF__;
 return(port4001 &1);
}

int T54X_xiof1_dir()
{
 __C54X_SET_XF__;
 return((port4001>>1) &1);
}


void T54X_set_xiof0(int v)
{
 __C54X_SET_XF__;
 if(v==OFF)  port4000=port4000 & ~1;
   else      port4000=port4000 |  1;
}

void T54X_set_xiof1(int v)
{
 __C54X_SET_XF__;
 if(v==OFF)  port4000=port4000 & ~2;
   else      port4000=port4000 |  2;
}

int T54X_xiof0()
{
 __C54X_SET_XF__;
 return(port4000 & 1);
}

int T54X_xiof1()
{
 __C54X_SET_XF__;
 return((port4000>>1) & 1);
}

/*--- XIO pins configuration control: T548/549/5402/5410 rev.2a and later */

void T54X_set_xio0_cnf(int cnf)
{
 __C54X_SET_XF__;
 if(cnf==T54X_XIO_IO_PIN)
     port4002=port4002 & ~1;
   else
     port4002=port4002 | 1;
}

void T54X_set_xio1_cnf(int cnf)	/* T5402 board only with HPI disabled */
{
 __C54X_SET_XF__;
 if(cnf==T54X_XIO_IO_PIN)
     port4002=port4002 & ~2;
   else
     port4002=port4002 | 2;
}

int T54X_xio0_cnf()
{
 __C54X_SET_XF__;
 return(port4002 &1);
}

int T54X_xio1cnf()
{
 __C54X_SET_XF__;
 return((port4002>>1) &1);
}


/********************* Data MEMORY Page Selector Control **************
  caution!: 
     this function is valid for TORNADO-548/549/5402/5410 DSP systems and
     is not available for TORNADO-542L DSP systems.
*/

void T54X_set_dmem_page(int p)
 /* Available PAGE# ('p') parameter range:
 	0..3 (T548/549/5410)
	0..1 (T5402)

    Each page is:
    	T548/T549/T5410: 32KW in the 0x8000..0xffff D-MEM address space.
        T5402:           48KW in the 0x4000..0xffff D-MEM address space.
  */

{
 __C54X_SET_XF__;
 port2000=(p&0x3); 
}

int T54X_dmem_page()
{
 __C54X_SET_XF__;
 return((port2000) &0x3);
}


/***************************************************************/

//======================================================================
#endif

#endif
