/***************************************************************************

  TORNADO-54x Host Control Utilities. (Host PC Environment) v.3.1
  (C) MicroLAB Systems Ltd, 1993-2001

****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <conio.h>
#include <fcntl.h>
#include <types.h>
#include <stat.h>
#include <io.h>
#include <stdlib.h>
#include <errno.h>
#include <dos.h>


#include <t54x_h.h>

static unsigned _t54x_id			= 0x0;	// T54x board ID
static unsigned _t54x_smp_segm		= 0x0;  // T54x SMP Window parameters
static unsigned _t54x_io_baddr		= 0x0;  // T54x ISA I/O base address
static unsigned _t54x_emu_io_baddr	= 0x0;	// UECM/ECC I/O base address


static unsigned _T54X_IO_BADDR_LIST[8]	= { 0x300,0x310,0x320,0x330,
					    0x340,0x350,0x360,0x370
					  };


static unsigned _T54X_SMP_SEGM_LIST[8]	= { 0x0000,0xb800,
					    0xc000,0xc800,
					    0xd000,0xd800,
					    0xe000,0xe800
					  };

static unsigned _T54X_ECC_IO_BADDR_LIST[8]	= { 0,0,0,0,0x240,0x280,0x320,0x340 };
static unsigned _T54X_UECM_IO_BADDR_LIST[4]	= { 0x240,0x280,0x320,0x340 };

static int _t54x_last_err = 0,	// last error code running T54X utilities
	   _t54x_err_flag = 0;	// sets to 1 in case error was found anywhere



#define _T54X_OPEN 	  		(_t54x_id!=0)

#define _T54X_VALID_BOARD 	(_t54x_id==T542L_DEV_ID || _t54x_id==T548_DEV_ID || _t54x_id==T549_DEV_ID || _t54x_id==T5402_DEV_ID || _t54x_id==T5410_DEV_ID || _t54x_id==T5416_DEV_ID)

#define _T542L_BOARD 	  	(_t54x_id==T542L_DEV_ID)
#define _T548_BOARD 	  	(_t54x_id==T548_DEV_ID)
#define _T549_BOARD 	  	(_t54x_id==T549_DEV_ID)
#define _T5402_BOARD 	  	(_t54x_id==T5402_DEV_ID)
#define _T5410_BOARD 	  	(_t54x_id==T5410_DEV_ID)
#define _T5416_BOARD 	  	(_t54x_id==T5416_DEV_ID)
#define	_T54X_FSEL_RG  	  	(_t54x_io_baddr+0x403)
#define	_T54X_FRG  	 	 	(_t54x_io_baddr+0x3)



static int _t54x_set_err(int e)	// local, used for exit fm SPI fucntions to set error code
 {
   if(e<0) _t54x_err_flag=1;
   _t54x_last_err=e;
   return(e);
 }



//------------------ T54X Utilities Setup functions ---------------------


int T54X_last_err()		// returns last function error
 {
   return(_t54x_last_err);
 }


int T54X_err_flag()		// returns trigger error flag
 {
   return(_t54x_err_flag);
 }


int T54X_clr_err_flag()		// clears trigger error flag
 {
   _t54x_err_flag=0;
 }


int T54X_last_err_message(char *s)	// interpretes last error message
 {
    if(s==NULL) return(T54X_OK);

    strcpy(s,"T54X>> \0");
    switch(_t54x_last_err)
      {
	case T54X_OK:
		strcat(s,"OK");
		break;

	case T54X_PARAM_ERR:
		strcat(s,"invalid function parameter");
		break;

	case T54X_SMP_OFF_ERR:
		strcat(s,"SMP I/F is OFF");
		break;

	case T54X_HW_ERR:
		strcat(s,"hardware error");
		break;

	case T54X_NOT_OPEN_ERR:
		strcat(s,"SPI is not open");
		break;

	case T54X_NOT_FOUND_ERR:
		strcat(s,"TORNADO-54x not found");
		break;

	case T54X_INV_CMND_ERR:
		strcat(s,"function is not allowed");
		break;

	default:
		strcat(s,"invalid error code");
		break;

      }
    return(T54X_OK);
 }

int T54X_open(unsigned io_addr)
     // open T54X board @IO_ADDR, identifies it and setup utilities vars
  {  int r;
     BYTE fsel_rg_old,n;

        _t54x_id=0;		// clear all vars
	_t54x_smp_segm=0;
	_t54x_io_baddr=0;
	_t54x_emu_io_baddr=0;

		// set error code and clear settings if incorrect IO_ADDR
	if(     (io_addr%16)!=0
             || io_addr<_T54X_IO_BADDR_LIST[0]
	     || io_addr>_T54X_IO_BADDR_LIST[T54X_IO_BADDR_CNF_QTY-1]
          )
	   {
	      r=T54X_PARAM_ERR;
	    hw_err:
              _t54x_id=0;
	      _t54x_smp_segm=0;
	      _t54x_io_baddr=0;
	      _t54x_emu_io_baddr=0;
	      return(_t54x_set_err(r));
	   }

              // now verify for this is a T54x board
	r=T54X_NOT_FOUND_ERR;
        _t54x_io_baddr=io_addr;
        fsel_rg_old=inp(_T54X_FSEL_RG);
        outp(_T54X_FSEL_RG,T54X_DEV_ID1_FRG);
	if(inp(_T54X_FSEL_RG)!=T54X_DEV_ID1_FRG)
	   {
             hw_err_restore_fsel_rg:
                outp(_T54X_FSEL_RG,fsel_rg_old);
	        goto hw_err;
	   }

		// read & verify device ID
	_t54x_id=(inp(_T54X_FRG)<<8) & 0xff00;	
	if(!_T54X_VALID_BOARD) goto hw_err_restore_fsel_rg;

		// read & verify SMP_SEGM
        r=T54X_HW_ERR;
        outp(_T54X_FSEL_RG,T54X_ISA_MI_BADDR_FRG);
	if(inp(_T54X_FSEL_RG)!=T54X_ISA_MI_BADDR_FRG)
        	goto hw_err_restore_fsel_rg;
        if((n=inp(_T54X_FRG))>=T54X_SMP_SEGM_CNF_QTY)
        	goto hw_err_restore_fsel_rg;
	_t54x_smp_segm=_T54X_SMP_SEGM_LIST[n];

		// read & verify ECC_BADDR for T542L board
        if(_T542L_BOARD)
	  { outp(_T54X_FSEL_RG,T54X_ECC_IO_BADDR_FRG);
    	    if(inp(_T54X_FSEL_RG)!=T54X_ECC_IO_BADDR_FRG)
        	   goto hw_err_restore_fsel_rg;
            if((n=inp(_T54X_FRG))>=T54X_ECC_IO_BADDR_CNF_QTY)
        	   goto hw_err_restore_fsel_rg;
	    _t54x_emu_io_baddr=_T54X_ECC_IO_BADDR_LIST[n];
	  }
	  else	// for T548/549/5402/5410 set default UECM I/O address
	  _t54x_emu_io_baddr=T54X_EMU_IO_BADDR_DEFAULT;

		// restore FSEL_RG data
        outp(_T54X_FSEL_RG,fsel_rg_old);
	if(inp(_T54X_FSEL_RG)!=fsel_rg_old)
        	   goto hw_err_restore_fsel_rg;

		// now exit OK
        r=T54X_OK;
        return(_t54x_set_err(r));
  }


int T54X_close()
     // close T54X SPI
  {
        _t54x_id=0;		// clear all vars
	_t54x_smp_segm=0;
	_t54x_io_baddr=0;
	_t54x_emu_io_baddr=0;
        return(_t54x_set_err(T54X_OK));
  }


unsigned T54X_io_baddr()		// return IO_BADDR
{ return(_t54x_io_baddr);
}

unsigned T54X_smp_segm()		// return SMP_SEGM
{ return(_t54x_smp_segm);
}

unsigned T54X_id()			// return DEV_ID
{ return(_t54x_id);
}

unsigned T54X_emu_io_baddr()		// return ECC_BADDR
{ return(_t54x_emu_io_baddr);
}


unsigned T54X_io_baddr_table(int i)    // returns 0 if error parameter
{
   if(i<0 || i>=T54X_IO_BADDR_CNF_QTY) return(0);
   return(_T54X_IO_BADDR_LIST[i]);
}

unsigned T54X_smp_segm_table(int i)
{
   if(i<0 || i>=T54X_SMP_SEGM_CNF_QTY) return(0);
   return(_T54X_SMP_SEGM_LIST[i]);
}

unsigned T54X_ecc_io_baddr_table(int i)
{
   if(i<0 || i>=T54X_ECC_IO_BADDR_CNF_QTY) return(0);
   return(_T54X_ECC_IO_BADDR_LIST[i]);
}


unsigned T54X_uecm_io_baddr_table(int i)
{
   if(i<0 || i>=T54X_UECM_IO_BADDR_CNF_QTY) return(0);
   return(_T54X_UECM_IO_BADDR_LIST[i]);
}


//-----------------------------------------------------------------------




//------------------ macros for FLAGs control/status --------------------

int T54X_set_fsel_rg_verif(unsigned frg)
     // Set flag sel register FRG# and verify.
  {
	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

        T54X_SET_FSEL_RG(frg);
	if(T54X_FSEL_RG_DATA!=frg)  return(_t54x_set_err(T54X_HW_ERR));
	return(_t54x_set_err(T54X_OK));
  }



int T54X_set_frg_verif(unsigned frg,BYTE value)
     // Set flag register FRG# value.
  { int r;
    BYTE fsel_old;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

        fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(frg))!=T54X_OK)
			return(_t54x_set_err(r));

        T54X_SET_FDATA_RG(value);
	if(T54X_FDATA_RG_DATA!=value)
			return(_t54x_set_err(T54X_HW_ERR));

	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	return(_t54x_set_err(T54X_OK));
  }


BYTE T54X_frg_data(unsigned frg)
     // read flag register value.
  { int r;
    BYTE value;
    BYTE fsel_old;

	if(!_T54X_OPEN)
		{ _t54x_set_err(T54X_NOT_OPEN_ERR);
                  return(0);
		}

        fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(frg))!=T54X_OK)
		{ _t54x_set_err(r);
		  return(0);
		}
        value=T54X_FDATA_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
		{ _t54x_set_err(r);
		  return(0);
		}
	_t54x_set_err(T54X_OK);
	return(value);
  }



BYTE T54X_sys_stat_frg_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN)
		{ _t54x_set_err(T54X_NOT_OPEN_ERR);
                  return(0);
		}

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_SET_HM_RQ_FRG))!=T54X_OK)
		{ _t54x_set_err(r);
		  return(0);
		}
	rgd = T54X_FDATA_RG_DATA & (  T54X_SB_ERR_FLAG_BITMASK
				    | T54X_MH_RQ_FLAG_BITMASK
				    | T54X_SB_ACK_FLAG_BITMASK
				    | T54X_HPI_HINT_FLAG_BITMASK
				    | T54X_HPI_ERR_FLAG_BITMASK
                                   );
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
		{ _t54x_set_err(r);
		  return(0);
		}
	_t54x_set_err(T54X_OK);
	return(rgd);
   }


int T54X_sb_err_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_SET_HM_RQ_FRG))!=T54X_OK)
			return(_t54x_set_err(r));

	rgd = T54X_FDATA_RG_DATA & T54X_SB_ERR_FLAG_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));

	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }


int T54X_sb_ack_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_SET_HM_RQ_FRG))!=T54X_OK)
			return(_t54x_set_err(r));

	rgd = T54X_FDATA_RG_DATA & T54X_SB_ACK_FLAG_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));

	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }


int T54X_mh_rq_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_SET_HM_RQ_FRG))!=T54X_OK)
			return(_t54x_set_err(r));

	rgd = T54X_FDATA_RG_DATA & T54X_MH_RQ_FLAG_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));

	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }

int T54X_hpi_hint_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_SET_HM_RQ_FRG))!=T54X_OK)
			return(_t54x_set_err(r));

	rgd = T54X_FDATA_RG_DATA & T54X_HPI_HINT_FLAG_BITMASK;

	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }

int T54X_hpi_err_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_SET_HM_RQ_FRG))!=T54X_OK)
			return(_t54x_set_err(r));

	rgd = T54X_FDATA_RG_DATA & T54X_HPI_ERR_FLAG_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));

	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }



int T54X_set_hm_rq()
  { BYTE fsel_old;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
        T54X_SET_FSEL_RG(T54X_SET_HM_RQ_FRG);
  	T54X_SET_FDATA_RG(0);
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	return(_t54x_set_err(T54X_OK));
   }


int T54X_clr_sb_err()
  { BYTE fsel_old;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	T54X_SET_FSEL_RG(T54X_CLR_SB_ERR_FRG);
	T54X_SET_FDATA_RG(0);
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	return(_t54x_set_err(T54X_OK));
   }


int T54X_clr_mh_rq()
  { BYTE fsel_old;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	T54X_SET_FSEL_RG(T54X_CLR_MH_RQ_FRG);
	T54X_SET_FDATA_RG(0);
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	return(_t54x_set_err(T54X_OK));
   }

int T54X_clr_hpi_err()
  { BYTE fsel_old;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	T54X_SET_FSEL_RG(T54X_CLR_HPI_ERR_FRG);
	T54X_SET_FDATA_RG(0);
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	return(_t54x_set_err(T54X_OK));
   }

int T54X_hpi_err_ie_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_HPI_IE_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & T54X_HPI_ERR_IE_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }

int T54X_hpi_hint_ie_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_HPI_IE_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & T54X_HPI_HINT_IE_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }

int T54X_hpi_disable_data()	// reads as HPI-ENABLE for !T5402
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_HPI_IE_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & T54X_HPI_DISABLE_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }


int T54X_hpi_ie_frg_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_HPI_IE_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & (T54X_HPI_ERR_IE_BITMASK | T54X_HPI_HINT_IE_BITMASK | T54X_HPI_DISABLE_BITMASK);
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd);
   }

int T54X_set_hpi_hint_ie(int i)
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

        if(i!=T54X_ON && i!=T54X_OFF)
			return(_t54x_set_err(T54X_PARAM_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_HPI_IE_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & ~T54X_HPI_HINT_IE_BITMASK;
	if(i!=T54X_OFF) rgd |= T54X_HPI_HINT_IE_BITMASK;
	if((r=T54X_set_frg_verif(T54X_HPI_IE_FRG,rgd))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(T54X_SET_FSEL_RG_VERIF(fsel_old));
   }

int T54X_set_hpi_err_ie(int i)
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

        if(i!=T54X_ON && i!=T54X_OFF)
			return(_t54x_set_err(T54X_PARAM_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_HPI_IE_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & ~T54X_HPI_ERR_IE_BITMASK;
	if(i!=T54X_OFF) rgd |= T54X_HPI_ERR_IE_BITMASK;
	if((r=T54X_set_frg_verif(T54X_HPI_IE_FRG,rgd))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(T54X_SET_FSEL_RG_VERIF(fsel_old));
   }

int T54X_set_hpi_disable(int i)	// !!! T5402 only
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));
	if(!_T5402_BOARD) return(_t54x_set_err(T54X_INV_CMND_ERR));

        if(i!=T54X_ON && i!=T54X_OFF)
			return(_t54x_set_err(T54X_PARAM_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_HPI_IE_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & ~T54X_HPI_DISABLE_BITMASK;
	if(i!=T54X_OFF) rgd |= T54X_HPI_DISABLE_BITMASK;
	if((r=T54X_set_frg_verif(T54X_HPI_IE_FRG,rgd))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(T54X_SET_FSEL_RG_VERIF(fsel_old));
   }



int T54X_dsp_stat_frg_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN)
		{ _t54x_set_err(T54X_NOT_OPEN_ERR);
                  return(0);
		}

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_DSP_STAT_FRG))!=T54X_OK)
		{ _t54x_set_err(r);
		  return(0);
		}
	rgd = T54X_FDATA_RG_DATA & (  T54X_DSP_MLOCK_BITMASK
				    | T54X_DSP_MRESET_BITMASK
				    | T54X_DSP_HPI_DISABLE_BITMASK
                                   );
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
		{ _t54x_set_err(r);
		  return(0);
		}
	_t54x_set_err(T54X_OK);
	return(rgd);
   }


int T54X_dsp_mlock_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_DSP_STAT_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & T54X_DSP_MLOCK_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }


int T54X_dsp_mreset_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_DSP_STAT_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & T54X_DSP_MRESET_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }

int T54X_dsp_hpi_disable_data()
  { BYTE fsel_old,rgd;
    int r;

	if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_DSP_STAT_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
	rgd = T54X_FDATA_RG_DATA & T54X_DSP_HPI_DISABLE_BITMASK;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));
	_t54x_set_err(T54X_OK);
	return(rgd!=0 ? 1:0);
   }


//------------------ macros/function for DEV_ID access --------------------

int T54X_dev_id_data(unsigned *dev_id)
     // read flag register value.
  { int r;
    BYTE lsb,msb,*b;
    BYTE fsel_old;

	if(!_T54X_OPEN)	 return(_t54x_set_err(T54X_NOT_OPEN_ERR));

	if(dev_id==NULL) return(_t54x_set_err(T54X_OK));

        fsel_old=T54X_FSEL_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_DEV_ID0_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
        lsb=T54X_FDATA_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(T54X_DEV_ID1_FRG))!=T54X_OK)
			return(_t54x_set_err(r));
        msb=T54X_FDATA_RG_DATA;
	if((r=T54X_SET_FSEL_RG_VERIF(fsel_old))!=T54X_OK)
			return(_t54x_set_err(r));

	b=(BYTE *)dev_id;
	*(b++)=lsb;
	*b=msb;
	return(_t54x_set_err(T54X_OK));
  }


int T54X_dev_name(char *s)	// returns string device name
  { int r;
    unsigned dev_id;

	if(s==NULL) return(_t54x_set_err(T54X_OK));
	if(!_T54X_OPEN)	 return(_t54x_set_err(T54X_NOT_OPEN_ERR));

        strcpy(s,"TORNADO-");
        switch(_t54x_id)
	   { case T542L_DEV_ID:
		 strcat(s,"542L");
		 break;

	     case T548_DEV_ID:
		 strcat(s,"548");
		 break;

	     case T549_DEV_ID:
		 strcat(s,"549");
		 break;

	     case T5402_DEV_ID:
		 strcat(s,"5402");
		 break;
	
	     case T5410_DEV_ID:
		 strcat(s,"5410");
		 break;

	     case T5416_DEV_ID:
		 strcat(s,"5416");
		 break;
	   }

	return(_t54x_set_err(T54X_OK));
  }


//========================== SMP ===============================


int T54X_set_smp_segm(unsigned segm)
     // Set SMP ISA-bus memory segment (baddr).
  {
     int i,r;

     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     for(i=0;i<T54X_SMP_SEGM_CNF_QTY;i++)
       { if(_T54X_SMP_SEGM_LIST[i]==segm)
	     {
	        if((r=T54X_SET_FRG_VERIF(T54X_ISA_MI_BADDR_FRG,(BYTE)i))!=T54X_OK)
			return(_t54x_set_err(r));
	        _t54x_smp_segm=segm;
		return(_t54x_set_err(T54X_OK));
	     }
       }
     return(_t54x_set_err(T54X_PARAM_ERR));
  }



int T54X_set_smp_off()
     // Set SMP off.
  {
     int r;

        if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

        if((r=T54X_SET_FRG_VERIF(T54X_ISA_MI_BADDR_FRG,0))!=T54X_OK)
			return(_t54x_set_err(r));
        _t54x_smp_segm=0;
	return(_t54x_set_err(T54X_OK));
  }


int T54X_smp_ptr(unsigned long addr,unsigned area,unsigned *page,void **p)
     // convert SRAM ADDR & AREA to *PAGE SMP_RG value & PC RAM **P pointer.

  {  int r;
     unsigned area_code;


     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     switch(area)
      {
        case T54X_PRAM_AREA:
	  if((addr/2L)>((_T542L_BOARD) ? T542L_PRAM_EADDR:T548_PRAM_EADDR) )
		return(_t54x_set_err(T54X_PARAM_ERR));
	  area_code=T54X_PRAM_AREA_SEL;
	  break;

        case T54X_DRAM_AREA:
	  if((addr/2L)>( (_T542L_BOARD) ? T542L_DRAM_EADDR:
	  		   ((_T5402_BOARD) ? T5402_XDRAM_EADDR:T548_XDRAM_EADDR)
                       )
	    )
		return(_t54x_set_err(T54X_PARAM_ERR));
	  area_code=T54X_DRAM_AREA_SEL;
	  break;

        case T54X_IO_AREA:
	  if((addr/2L)>T54X_IO_EADDR)
		return(_t54x_set_err(T54X_PARAM_ERR));
	  area_code=T54X_IO_AREA_SEL;
	  break;

        default:
		return(_t54x_set_err(T54X_PARAM_ERR));
      }

     *page=T54X_MAKE_SMP(addr) | area_code;
     if(p!=NULL) *p=T54X_MAKE_SMP_PTR(addr);
     return(_t54x_set_err(T54X_OK));
   }


int T54X_set_smp_ptr(unsigned long addr,unsigned area,void **p)
     // Set SMP RG to the specified SRAM address.
     // Set PC SMP pointer to the *P.

  {  unsigned page;
     int r;

     if((r=T54X_smp_ptr(addr,area,&page,p))!=T54X_OK)
		return(_t54x_set_err(r));
     T54X_SET_SMP_RG(page);
     return(_t54x_set_err(T54X_OK));
   }

//========================================================================




//--------------- macros/function for UECM/ECC access --------------------

int T54X_emu_io_baddr_data()
     // Returns current EMU I/O base address
  {
     int r;

     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     return(_t54x_emu_io_baddr);
  }




int T54X_set_emu_io_baddr(unsigned io_addr)
     // Activate and Set ISA-bus I/O address for ECC/T542L
     // or set I/O address for UECM/T548.
  {
     int i,r;

     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     if(_T542L_BOARD)
      { for(i=0;i<T54X_ECC_IO_BADDR_CNF_QTY;i++)
         { if(_T54X_ECC_IO_BADDR_LIST[i]==io_addr)
	     {	if((r=T54X_SET_FRG_VERIF(T54X_ECC_IO_BADDR_FRG,(BYTE)i))!=T54X_OK)
			return(_t54x_set_err(r));
	        _t54x_emu_io_baddr=io_addr;
	        return(_t54x_set_err(T54X_OK));
	     }
          }
        return(_t54x_set_err(T54X_PARAM_ERR));
      }
      else
      {
	for(i=0;i<T54X_UECM_IO_BADDR_CNF_QTY;i++)
         { if(_T54X_UECM_IO_BADDR_LIST[i]==io_addr)
	     {	_t54x_emu_io_baddr=io_addr;
	        return(_t54x_set_err(T54X_OK));
	     }
          }
        return(_t54x_set_err(T54X_PARAM_ERR));
      }
  }



int T54X_set_emu_reset()
     // Set UECM/ECC reset.
  {
     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     if(_T542L_BOARD)
	{ if(_t54x_emu_io_baddr!=0)
	    { outp(_t54x_emu_io_baddr+0x818,0);
              return(_t54x_set_err(T54X_OK));
	    }
	   else
              return(_t54x_set_err(T54X_INV_CMND_ERR));
	}
       else
	{
	  if(_t54x_emu_io_baddr!=0)
	   {	outp(_t54x_emu_io_baddr+0x818,0);
	        return(_t54x_set_err(T54X_OK));
	   }
	   else
	     return(_t54x_set_err(T54X_INV_CMND_ERR));
	}
  }

int T54X_set_emu_xdsp()
     // Set UECM/T548 to external DSP or set ECC/T542L off.
  {
     int r;

     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     if(_T542L_BOARD)
       { if((r=T54X_SET_FRG_VERIF(T54X_ECC_IO_BADDR_FRG,0))!=T54X_OK)
			return(_t54x_set_err(r));
        _t54x_emu_io_baddr=0;
        return(_t54x_set_err(T54X_OK));
       }
      else	// T548/548/5402/5410: select XDSP
       {
	  if(_t54x_emu_io_baddr!=0)
		{  outp(_t54x_emu_io_baddr+0x801,0);
		  return(_t54x_set_err(T54X_OK));
		}
	   else
	     return(_t54x_set_err(T54X_INV_CMND_ERR));
       }
  }

int T54X_set_emu_mdsp()
     // Set UECM@T548/549/5402/5410 to on-board DSP
  {
     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     if(_T542L_BOARD)
       { if(_t54x_emu_io_baddr!=0)
                 return(_t54x_set_err(T54X_OK));
           else  return(_t54x_set_err(T54X_INV_CMND_ERR));
       }
      else	// T548/549/5402/5410: select MDSP
       {
	if(_t54x_emu_io_baddr!=0)
	  {	outp(_t54x_emu_io_baddr+0x800,0);
		return(_t54x_set_err(T54X_OK));
	  }
	   else
	     return(_t54x_set_err(T54X_INV_CMND_ERR));
       }
  }

int T54X_emu_path_data()
     // Returns XPATH/MPATH for UECM@T548/549/5402/5410/5416 and MPATH for ECC/T542L.
  {
     int r;

     if(!_T54X_OPEN) return(_t54x_set_err(T54X_NOT_OPEN_ERR));

     if(_T542L_BOARD)
       {
        if(_t54x_emu_io_baddr==0)
	     return(_t54x_set_err(T54X_INV_CMND_ERR));
        return(_t54x_set_err(T54X_EMU_MDSP));
       }
      else	// T548/549/5402/5410: return EMU PATH SEL
       {
	  if(_t54x_emu_io_baddr!=0)
	     return((int) (inp(_t54x_emu_io_baddr+0x800) & 0x1));
	   else
	     return(_t54x_set_err(T54X_INV_CMND_ERR));
       }
  }
